<?php

declare(strict_types=1);

namespace mirzaev\zkmr\calculator\controllers;

use Exception;
use mirzaev\zkmr\calculator\controllers\core;
use mirzaev\zkmr\calculator\models\calculators_model as calculators;
use mirzaev\zkmr\calculator\models\settings_model as settings;
use mirzaev\zkmr\calculator\models\fences_model as fences;

use Twig\Loader\FilesystemLoader;
use Twig\Environment as view;

/**
 * Контроллер основной страницы
 *
 * @package mirzaev\zkmr\calculator\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class calculator_controller extends core
{
  /**
   * Калькулятор
   *
   * HTML-код с калькулятором
   *
   * @param array $vars Параметры
   */
  public function index(array $vars = []): ?string
  {
    // Генерация представления
    return $this->view->render(DIRECTORY_SEPARATOR . 'calculator' . DIRECTORY_SEPARATOR . 'index.html', $vars);
    }

    /**
     * Модуль: "тип пользователя"
     *
     * HTML-код с кнопками: "физическое лицо" и "юридическое лицо"
     *
     * @param array $vars Параметры
     */
    public function buyer(array $vars = []): ?string
    {
      // Инициализация параметров
      $vars['buyer'] = $vars['value'] ?? 'individual';

      // Удаление параметров
      unset($vars['value']);

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'buyer.html', $vars);
    }

    /**
     * Модуль: "сложность"
     *
     * HTML-код с кнопками: "легко", "средне" и "сложно"
     *
     * @param array $vars Параметры
     */
    public function complexity(array $vars = []): ?string
    {
      // Инициализация параметров
      $vars['complexity'] = $vars['value'] ?? 'medium';

      // Удаление параметров
      unset($vars['value']);

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'complexity.html', $vars);
    }

    /**
     * Модуль: "меню"
     *
     * HTML-код с кнопками добавления калькуляторов
     *
     * @param array $vars Параметры
     */
    public function menu(array $vars = []): ?string
    {
      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'menu.html', $vars);
    }

    /**
     * Модуль: "результат"
     *
     * HTML-код с данными результата калькуляции
     *
     * @param array $vars Параметры
     */
    public function result(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      // Инициализация данных калькулятора
      $vars['discount'] = settings::read('discount', $vars['errors']['calculators']);

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'result.html', $vars);
    }

    /**
     * Модуль: "тип забора"
     *
     * HTML-код со списком типов забора
     *
     * @param array $vars Параметры
     */
    public function fence(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'fences' . DIRECTORY_SEPARATOR . $vars['type'] . '.html', $vars);
    }

    /**
     * Модуль: "разделитель"
     *
     * HTML-код с разделителем элементов
     *
     * @param array $vars Параметры
     */
    public function divider(array $vars = []): ?string
    {
      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'divider.html', $vars);
    }

    /**
     * Профнастил 
     *
     * HTML-код с калькулятором профнастила
     *
     * @param array $vars Параметры
     */
    public function profnastil(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      // Инициализация данных калькулятора
      $vars['calculators'] = ['profnastil' => [
        'width' => (int) round((float) settings::read('default_width', $vars['errors']['calculators'])),
        'height' => (int) round((float) settings::read('default_height', $vars['errors']['calculators'])),
        'length' => (int) round((float) settings::read('default_length', $vars['errors']['calculators'])),
        'amount' => (int) round((float) settings::read('default_amount', $vars['errors']['calculators'])),
        'type' => settings::read('default_type', $vars['errors']['calculators']),
        'holes' => (int) round((float) settings::read('default_holes', $vars['errors']['calculators'])),
        'diameter' => (float) settings::read('default_diameter', $vars['errors']['calculators'])
        ]];

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'profnastil.html', $vars);
    }

    /**
     * 3D-сетка 
     *
     * HTML-код с калькулятором 3D-сетки
     *
     * @param array $vars Параметры
     */
    public function setka(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      // Инициализация данных калькулятора
      $vars['calculators'] = ['profnastil' => [
        'width' => (int) round((float) settings::read('default_width', $vars['errors']['calculators'])),
        'height' => (int) round((float) settings::read('default_height', $vars['errors']['calculators'])),
        'length' => (int) round((float) settings::read('default_length', $vars['errors']['calculators'])),
        'amount' => (int) round((float) settings::read('default_amount', $vars['errors']['calculators'])),
        'type' => settings::read('default_type', $vars['errors']['calculators']),
        'holes' => (int) round((float) settings::read('default_holes', $vars['errors']['calculators'])),
        'diameter' => (float) settings::read('default_diameter', $vars['errors']['calculators'])
        ]];

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'setka.html', $vars);
    }

    /**
     * Евроштакет
     *
     * HTML-код с калькулятором евроштакета
     *
     * @param array $vars Параметры
     */
    public function evroshtaket(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      // Инициализация данных калькулятора
      $vars['calculators'] = ['profnastil' => [
        'width' => (int) round((float) settings::read('default_width', $vars['errors']['calculators'])),
        'height' => (int) round((float) settings::read('default_height', $vars['errors']['calculators'])),
        'length' => (int) round((float) settings::read('default_length', $vars['errors']['calculators'])),
        'amount' => (int) round((float) settings::read('default_amount', $vars['errors']['calculators'])),
        'type' => settings::read('default_type', $vars['errors']['calculators']),
        'holes' => (int) round((float) settings::read('default_holes', $vars['errors']['calculators'])),
        'diameter' => (float) settings::read('default_diameter', $vars['errors']['calculators'])
        ]];

      // Генерация представления
      return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'evroshtaket.html', $vars);
    }

    /**
     * Расчёт
     *
     * Генерирует ответ в виде ['expenses' => 0, 'income' => 0, 'profit' => 0]
     *
     * @param array $vars Параметры
     *
     * @todo
     * 5. Убрать передачу цены работы (оставить только время работы в часах и цену за работу в час)
     */
    public function calculate(array $vars = []): ?string
    {
      // Инициализация журнала ошибок
      $vars['errors'] = ['calculators' => []];

      try {
        // Инициализация параметров из тела запроса (подразумевается, что там массивы с параметрами)
        $vars['input'] = json_decode(file_get_contents('php://input'), true);

        $calculators = $vars['input']['calculators'];
        $discount = $vars['input']['discount'];
        $cutting = $vars['input']['cutting'];

        // Инициализация переменных для буфера вывода
        $machines = $managers = $engineers = $operators = $handymans = $other = 0;

        if (count($calculators) > 0) {
          // Найдены калькуляторы

          foreach ($calculators as $i => $calculator) {
            // Перебор калькуляторов

            foreach (['calculator'] as &$parameter) {
              // Перебор мета-параметров

              // Инициализация общего параметра
              $type =  $calculator[$parameter];

              // Инициализация параметра для обработчика калькулятора
              unset($calculator[$parameter]);
                    }

                    // Инициализация номера калькулятора в его категории
                    $number = count($vars['errors']['calculators'][$type] ?? []);

                    // Инициализация журнала ошибок для калькулятора
                    $calculator['errors'] = [];

                    // Инициализация журнала ошибок для буфера вывода
                    $vars['errors']['calculators'][$type][$number] = &$calculator['errors'];

                    // Инициализация буфера параметров
                    $parameters = [];

                    // Инициализация параметра типа покупателя (подразумевается, что если не "entity", то "individual")
                    $parameters['company'] = $calculator['buyer'] === 'entity';
                    unset($calculator['buyer']);

                    // Перенос остальных параметров в буфер параметров
                    $parameters += $calculator;

                    // Расчёт
                    [$machines, $managers, $engineers, $operators, $handymans, $other] = calculators::$type(...$parameters + ['cutting' => $cutting]);
                }
            } else {
              // Не найдены калькуляторы

              throw new exception('Не найдены калькуляторы');
            }
        } catch (exception $e) {
          // Запись в журнал ошибок
          $vars['errors']['calculators'][] = [
            'text' => $e->getMessage(),
            'file' => $e->getFile(),
            'line' => $e->getLine(),
            'stack' => $e->getTrace()
            ];
        }


        return json_encode([
          'machines' => $machines,
          'managers' => $managers,
          'engineers' => $engineers,
          'operators' => $operators,
          'handymans' => $handymans,
          'other' => $other + ['discount' => $discount],
          'errors' => $vars['errors']
        ]);
    }
}
