"use strict";

const calculator = {
  index: document.getElementById("calculator"),
  calculators: [],
  account: [],
  settings: {
    defaults: {
    },
  },
  init() {
    // Инициализация калькулятора

    this.generate.menu()
      .then(
        () => {
          this.authenticate(cookie.read("id"))
            .then(
              (success) => {
                // Запись данных аккаунта
                this.account = success;

                if (
                  this.account !== undefined &&
                  typeof this.account === "object" &&
                  this.account.permissions !== undefined
                ) {
                  // Найден аккаунт

                  if (this.account.permissions.calculate == 1) {
                    // Разрешено использовать калькулятор

                    this.generate.result();
                  }
                }
              },
            );
        },
      );

    console.log("[КАЛЬКУЛЯТОР] Инициализирован");
  },
  async authenticate(id) {
    // Запрос и генерация HTML с данными о типе покупателя (юр. лицо и физ. лицо)'

    return await fetch("/account/data?id=" + id, {
      method: "POST",
      headers: { "content-type": "application/x-www-form-urlencoded" },
    }).then((response) => {
      if (response.status === 200) {
        return response.json().then(
          (success) => {
            console.log("[КАЛЬКУЛЯТОР] Загружены данные пользователя: " + id);

            return success;
          },
          () => {
            console.log(
              "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить данные пользователя: " +
                id,
            );
          },
        );
      }
    });
  },
  async calculate() {
    // Запрос и генерация HTML с данными о рассчете со всех калькуляторов

    // Инициализация параметров
    const cutting = document.getElementById("cutting");
    const discount = document.getElementById("discount");

    // Инициализация буфера запроса
    const query = {
      calculators: {},
      cutting: +cutting.value ?? 0,
      discount: +discount.value ?? 0,
    };

    for (const number in this.calculators) {
      // Перебор калькуляторов

      // Инициализация буфера запроса для нового калькулятора
      query["calculators"][number] = {};

      // Инициализация типа калькулятора
      query["calculators"][number]["calculator"] = this.calculators[number]
        .getAttribute("data-calculator");

      for (const buyer of this.index.querySelectorAll('input[name="buyer"]')) {
        // Перебор полей с параметрами типа заказчика

        if (buyer.checked) {
          // Найдено выбранное поле

          // Запись в буфер запроса
          query["calculators"][number]["buyer"] = buyer.value;
        }
      }

      for (
        const complexity of this.index.querySelectorAll(
          'input[name="complexity"]',
        )
      ) {
        // Перебор полей с параметрами сложности

        if (complexity.checked) {
          // Найдено выбранное поле

          // Запись в буфер запроса
          query["calculators"][number]["complexity"] = complexity.value;
        }
      }

      for (
        const field of this.calculators[number].querySelectorAll(
          "[data-calculator-parameter]",
        )
      ) {
        // Перебор полей с параметрами

        if (field.getAttribute("type") === "checkbox") {
          // Флажок

          // Запись в буфер запроса
          query["calculators"][number][
            field.getAttribute("data-calculator-parameter")
          ] = field.checked;
        } else if (
          field.getAttribute("type") === "text" ||
          field.getAttribute("type") === "number" ||
          field.getAttribute("type") === "range"
        ) {
          // Текстовое, цифровое поле или ползунок

          // Запись в буфер запроса
          query["calculators"][number][
            field.getAttribute("data-calculator-parameter")
          ] = field.value;
        } else {
          // Элемент с тегом <select> (подразумевается)

          // Запись в буфер запроса
          query["calculators"][number][
            field.getAttribute("data-calculator-parameter")
          ] = field.value ?? field.options[field.selectedIndex].text;
        }
      }
    }

    return await fetch("/calculator/calculate", {
      method: "POST",
      headers: { "content-type": "application/json" },
      body: JSON.stringify(query),
    }).then((response) => {
      if (response.status === 200) {
        response.json().then(
          (success) => {
            // Инициализация буфера расходов
            let expenses = 0;

            // Инициализация буфера с данными расчёта
            let result;

            if (this.generate.error(success.errors) > 0) {
              // Найдены ошибки

              // Генерация текста ответа
              result = "Ошибка";
            } else {
              // Не найдены ошибки

              if (success.other.cutting !== undefined) {
                // Получены данные времени работы

                // Запись полученных данных
                cutting.value = success.other.cutting;
                cutting.parentElement.children[0].innerText =
                  "Длина реза 1 детали (" + cutting.value + "мм)";

                // Разблокировка параметра
                cutting.removeAttribute("disabled");
              }

              if (success.other.discount !== undefined) {
                // Получены данные скидки

                // Запись полученных данных
                discount.value = success.other.discount;
                discount.parentElement.children[0].innerText = "Скидка (" +
                  discount.value + "%)";
              }

              for (const [, machine] of Object.entries(success.machines)) {
                // Перебор станков

                // Прибавление данных станка к буферу расходов
                expenses += machine.electricity + (machine.metal ?? 0);

                // Прибавление амортизации к буферу вывода
                expenses += machine.reprocessing ?? 0;

                // Прибавление линз к буферу вывода
                expenses += machine.lenses ?? 0;
              }

              for (const [, manager] of Object.entries(success.managers)) {
                // Перебор менеджеров

                // Прибавление данных менеджера к буферу расходов
                expenses += manager.time * manager.hour;
              }

              for (const [, engineer] of Object.entries(success.engineers)) {
                // Перебор инженеров

                // Прибавление данных инженера к буферу расходов
                expenses += engineer.time * engineer.hour;
              }

              for (const [, operator] of Object.entries(success.operators)) {
                // Перебор операторов

                // Прибавление данных оператора к буферу расходов
                expenses += (operator.time.design + operator.time.machine) *
                  operator.hour;
              }

              for (const [, handyman] of Object.entries(success.handymans)) {
                // Перебор разнорабочих

                // Прибавление данных к буферу расходов
                expenses += handyman.time * handyman.hour;
              }

              // Прибавление аренды к буферу расходов
              expenses += success.other.rent ?? 0;

              // Прибавление переработки к буферу расходов
              expenses += success.other.reprocessing ?? 0;

              // Прибавление баллонов к буферу расходов
              expenses += success.other.baloons.cost ?? 0;

              // Вычисление наценки (коэффициент)
              expenses *= success.other.additive ?? 1;

              // Вычитание скидки менеджера
              expenses -= expenses * ((discount.value ?? 100) / 100);

              // Округление
              expenses = expenses.toFixed(2);

              // Генерация текста ответа
              result = expenses + " рублей";
            }

            if (this.generate.result(result)) {
              console.log(
                `[КАЛЬКУЛЯТОР] Сгенерирован результат: ${expenses} рублей`,
              );
            } else {
              console.log(
                "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось сгенерировать результат",
              );
            }
          },
          () => {
            console.log(
              "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось сгенерировать результат",
            );
          },
        );
      }
    });
  },
  generate: {
    async menu() {
      // Запрос и генерация HTML с кнопками добавления калькулятора

      return await fetch("/calculator/generate/menu", {
        method: "POST",
        headers: { "content-type": "application/x-www-form-urlencoded" },
      }).then((response) => {
        if (response.status === 200) {
          response.text().then(
            (success) => {
              calculator.index.insertAdjacentHTML("beforeend", success);

              console.log(
                "[КАЛЬКУЛЯТОР] Загружен элемент с кнопками добавления калькулятора",
              );
            },
            () => {
              console.log(
                "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с кнопками добавления калькулятора",
              );
            },
          );
        }
      });
    },
    async fence(element, type = 1) {
      // Генерация полей с выбором типа забора

      return await fetch("/calculator/generate/fence?type=" + type, {
        method: "POST",
        headers: { "content-type": "application/x-www-form-urlencoded" },
      }).then((response) => {
        if (response.status === 200) {
          response.text().then(
            (success) => {
              // Поиск оболочки списка
              const wrap = element.querySelectorAll('select[name="fence"]')[0];

              // Запись полученного списка в оболочку
              wrap.parentElement.parentElement.insertAdjacentHTML(
                "afterend",
                success,
              );

              // Удаление старого списка
              if (wrap !== undefined) wrap.parentElement.parentElement.remove();

              console.log("[КАЛЬКУЛЯТОР] Загружен список с типами забора");
            },
            () => {
              console.log(
                "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить список с типами забора",
              );
            },
          );
        }
      });
    },
    async divider(element, position) {
      // Запрос и генерация HTML с данными о результате калькуляции

      return await fetch("/calculator/generate/divider", {
        method: "POST",
        headers: { "content-type": "application/x-www-form-urlencoded" },
      }).then((response) => {
        if (response.status === 200) {
          response.text().then(
            (success) => {
              if (element === undefined || position === undefined) {
                // Не задан элемент и позиция добавляемого разделителя

                // Запись разделителя в конце калькулятора
                calculator.index.insertAdjacentHTML("beforeend", success);
              } else {
                // Задан элемент и позиция добавляемого разделителя

                // Запись разделителя по заданным параметрам
                element.insertAdjacentHTML(position, success);
              }

              console.log("[КАЛЬКУЛЯТОР] Загружен разделитель");
            },
            () => {
              console.log(
                "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить разделитель",
              );
            },
          );
        }
      });
    },
    result(expenses) {
      // Запрос и генерация HTML с данными о результате калькуляции

      async function request() {
        return await fetch("/calculator/generate/result", {
          method: "POST",
          headers: { "content-type": "application/x-www-form-urlencoded" },
        }).then((response) => {
          if (response.status === 200) {
            response.text().then(
              (success) => {
                calculator.index.insertAdjacentHTML("beforeend", success);

                console.log(
                  "[КАЛЬКУЛЯТОР] Загружен элемент с данными о результате калькуляции",
                );
              },
              () => {
                console.log(
                  "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось загрузить элемент с данными о результате калькуляции",
                );
              },
            );
          }
        });
      }

      if (document.getElementById("result") === null) {
        // Не найден элемент с данными расчётов
      } else {
        // Найден элемент с данными расчётов

        if (expenses !== undefined) {
          // Переданы расходы

          // Инициализация элемента
          const element = document.getElementById("calculate");

          if (element == null) {
            // Не найден элемент с результатом расчёта

            return false;
          }

          // Запись расходов в элемент (подразумевается кнопка отправки на расчёт)
          element.innerText = expenses;

          return true;
        }
      }

      return request();
    },
    error(errors = []) {
      // Генерация ошибки

      // Инициализация количества обработанных ошибок
      let amount = 0;

      if (typeof errors === "object") {
        // Передан массив с ошибками и он является массивом

        // Инициализация буфера для проверки вложенности массива
        const first = Object.values(errors)[0];

        if (first !== undefined && first.text === undefined) {
          // Найден массив с ошибками (категория)

          // Вход в рекурсию
          amount += this.error(first);
        } else {
          // Не найден массив с ошибками (подразумевается, что это и есть информация об ошибке)

          // Инициализация элемента-оболочки
          const list = document.getElementById("errors");

          // Перезапись данных об ошибках
          list.innerText = "";

          // Проверка на наличие ошибок
          if (errors.length === 0) return false;

          if (list !== null) {
            // Оболочка найдена

            for (const [, error] of Object.entries(errors)) {
              // Перебор станков

              // Инициализация элемента-заголовка
              const term = document.createElement("dt");

              // Запись содержимого
              term.innerText = error.text;

              // Инициализация элемента-описания
              const definition = document.createElement("dd");

              // Запись содержимого
              definition.innerText = error.file + " в строке " + error.line;

              // input.setAttribute('id', element.id);

              // Запись в список
              list.insertAdjacentElement("beforeend", term);
              list.insertAdjacentElement("beforeend", definition);

              // Добавление к счётчику обработанных ошибок
              ++amount;
            }
          }
        }
      }

      return amount;
    },
    calculators: {
      async profnastil() {
        // Запрос и генерация HTML с калькулятором лазерной резки

        function write(target, position, html) {
          if (
            target === undefined || position === undefined || html === undefined
          ) return false;

          // Запись калькулятора после последнего калькулятора
          target.insertAdjacentHTML(position, html);

          // Поиск калькуляторов
          const calculators = calculator.index.querySelectorAll(
            "section[data-calculator]",
          );

          // Инициализация идентификатора калькулятора
          const id = calculators.length - 1;

          // Запись калькулятору его идентификатора
          calculators[id].id = "profnastil_" + id;

          // Запись в реестр последнего калькулятора (подразумевается, что он новый и только что был записан)
          calculator.calculators.push(calculators[id]);

          // Запись в журнал
          console.log(
            "[КАЛЬКУЛЯТОР] Инициализирован калькулятор лазерной резки",
          );
        }

        return await fetch("/calculator/generate/profnastil", {
          method: "POST",
          headers: { "content-type": "application/x-www-form-urlencoded" },
        }).then((response) => {
          if (response.status === 200) {
            response.text().then(
              (success) => {
                // Поиск последнего калькулятора
                const last =
                  calculator.calculators[calculator.calculators.length - 1];

                if (last !== undefined && last !== null) {
                  // Найден калькулятор

                  // Инициализация разделителя перед калькулятором 
                  calculator.generate.divider(last, "afterend").then(
                    () => write(last, "afterend", success),
                  );
                } else {
                  // Не найден калькулятор

                  calculator.generate.divider(menu, "beforebegin").then(
                    () => {
                      // Поиск меню
                      const menu = document.getElementById("menu");

                      if (menu !== null) {
                        // Найдено меню

                        // Инициализация разделителя перед меню
                        write(menu, "beforebegin", success)
                      } else {
                        // Не найдено меню

                        // Поиск результатов калькуляции
                        const result = document.getElementById("result");

                        if (result !== null) {
                          // Найден элемент с результатами калькуляции

                          // Инициализация разделителя перед меню
                          calculator.generate.divider(result, "beforebegin")
                            .then((result) =>
                              write(result, "beforebegin", success)
                            );
                        } else {
                          // Не найден элемент с результатами калькуляции

                          // Инициализация разделителя перед меню
                          calculator.generate.divider().then(() =>
                            write(calculator.index, "beforeend", success)
                          );
                        }
                      }
                    },
                  );
                }
              },
              () => {
                // Запись в журнал
                console.log(
                  "[КАЛЬКУЛЯТОР] [ОШИБКА] Не удалось инициализировать калькулятор лазерной резки",
                );
              },
            );
          }
        });
      },
    },
  },
};
